@@ -7,6 +7,7 @@
 #include <jellyfish/err.hpp>
 #include <jellyfish/cooperative_pool2.hpp>
 #include <jellyfish/cpp_array.hpp>
+#include <jellyfish/parser_common.hpp>
 
 namespace jellyfish {
 struct header_sequence_qual {
@@ -22,8 +23,6 @@ struct sequence_list {
 template<typename StreamIterator>
 class whole_sequence_parser : public jellyfish::cooperative_pool2<whole_sequence_parser<StreamIterator>, sequence_list> {
   typedef jellyfish::cooperative_pool2<whole_sequence_parser<StreamIterator>, sequence_list> super;
-  typedef std::unique_ptr<std::istream> stream_type;
-  enum file_type { DONE_TYPE, FASTA_TYPE, FASTQ_TYPE };
 
   struct stream_status {
     file_type   type;
@@ -70,11 +69,16 @@ public:
     case FASTQ_TYPE:
       read_fastq(st, buff);
       break;
+#ifdef HAVE_HTSLIB
+    case SAM_TYPE:
+      read_sam(st, buff);
+      break;
+#endif
     case DONE_TYPE:
       return true;
     }
 
-    if(st.stream->good())
+    if(st.stream.good())
       return false;
 
     // Reach the end of file, close current and try to open the next one
@@ -88,8 +92,8 @@ public:
 protected:
   void open_next_file(stream_status& st) {
     st.stream.reset();
-    st.stream = streams_iterator_.next();
-    if(!st.stream) {
+    st.stream = std::move(streams_iterator_.next());
+    if(!st.stream.good()) {
       st.type = DONE_TYPE;
       return;
     }
@@ -97,32 +101,43 @@ protected:
     ++files_read_;
     // Update the type of the current file and move past first header
     // to beginning of sequence.
-    switch(st.stream->peek()) {
-    case EOF: return open_next_file(st);
-    case '>':
-      st.type = FASTA_TYPE;
-      break;
-    case '@':
-      st.type = FASTQ_TYPE;
-      break;
-    default:
-      throw std::runtime_error("Unsupported format"); // Better error management
+    if(st.stream.standard) {
+      switch(st.stream.standard->peek()) {
+      case EOF: return open_next_file(st);
+      case '>':
+        st.type = FASTA_TYPE;
+        break;
+      case '@':
+        st.type = FASTQ_TYPE;
+        break;
+      default:
+        throw std::runtime_error("Unsupported format"); // Better error management
+      }
+    }
+#ifdef HAVE_HTSLIB
+    else if(st.stream.sam) {
+      st.type = SAM_TYPE;
+    }
+#endif
+    else {
+      st.type = DONE_TYPE;
     }
   }
 
   void read_fasta(stream_status& st, sequence_list& buff) {
     size_t&      nb_filled = buff.nb_filled;
     const size_t data_size = buff.data.size();
+    auto&        stream    = *st.stream.standard;
 
-    for(nb_filled = 0; nb_filled < data_size && st.stream->peek() != EOF; ++nb_filled) {
+    for(nb_filled = 0; nb_filled < data_size && stream.peek() != EOF; ++nb_filled) {
       ++reads_read_;
       header_sequence_qual& fill_buff = buff.data[nb_filled];
-      st.stream->get(); // Skip '>'
-      std::getline(*st.stream, fill_buff.header);
+      stream.get(); // Skip '>'
+      std::getline(stream, fill_buff.header);
       fill_buff.seq.clear();
-      for(int c = st.stream->peek(); c != '>' && c != EOF; c = st.stream->peek()) {
-        std::getline(*st.stream, st.buffer); // Wish there was an easy way to combine the
-        fill_buff.seq.append(st.buffer);             // two lines avoiding copying
+      for(int c = stream.peek(); c != '>' && c != EOF; c = stream.peek()) {
+        std::getline(stream, st.buffer); // Wish there was an easy way to combine the
+        fill_buff.seq.append(st.buffer); // two lines avoiding copying
       }
     }
   }
@@ -130,31 +145,63 @@ protected:
   void read_fastq(stream_status& st, sequence_list& buff) {
     size_t&      nb_filled = buff.nb_filled;
     const size_t data_size = buff.data.size();
+    auto&        stream    = *st.stream.standard;
 
-    for(nb_filled = 0; nb_filled < data_size && st.stream->peek() != EOF; ++nb_filled) {
+    for(nb_filled = 0; nb_filled < data_size && stream.peek() != EOF; ++nb_filled) {
       ++reads_read_;
       header_sequence_qual& fill_buff = buff.data[nb_filled];
-      st.stream->get(); // Skip '@'
-      std::getline(*st.stream, fill_buff.header);
-      fill_buff.seq.clear();
-      while(st.stream->peek() != '+' && st.stream->peek() != EOF) {
-        std::getline(*st.stream, st.buffer); // Wish there was an easy way to combine the
+      stream.get(); // Skip '@'
+      std::getline(stream, fill_buff.header);
+
+      if(stream.peek() != '+')
+        std::getline(stream, fill_buff.seq);
+      else
+        fill_buff.seq.clear();
+      while(stream.peek() != '+' && stream.peek() != EOF) {
+        std::getline(stream, st.buffer); // Wish there was an easy way to combine the
         fill_buff.seq.append(st.buffer);             // two lines avoiding copying
       }
-      if(!st.stream->good())
+      if(!stream.good())
         throw std::runtime_error("Truncated fastq file");
-      st.stream->ignore(std::numeric_limits<std::streamsize>::max(), '\n');
-      fill_buff.qual.clear();
-      while(fill_buff.qual.size() < fill_buff.seq.size() && st.stream->good()) {
-        std::getline(*st.stream, st.buffer);
+      stream.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
+      if(stream.peek() != '+')
+        std::getline(stream, fill_buff.qual);
+      else
+        fill_buff.qual.clear();
+      while(fill_buff.qual.size() < fill_buff.seq.size() && stream.good()) {
+        std::getline(stream, st.buffer);
         fill_buff.qual.append(st.buffer);
       }
       if(fill_buff.qual.size() != fill_buff.seq.size())
         throw std::runtime_error("Invalid fastq file: wrong number of quals");
-      if(st.stream->peek() != EOF && st.stream->peek() != '@')
+      if(stream.peek() != EOF && stream.peek() != '@')
         throw std::runtime_error("Invalid fastq file: header missing");
+      // std::cerr << nb_filled << ": " << fill_buff.seq << '\n'
+      //           << nb_filled << ": " << fill_buff.qual << '\n';
     }
   }
+
+#ifdef HAVE_HTSLIB
+  void read_sam(stream_status& st, sequence_list& buff) {
+    size_t&      nb_filled = buff.nb_filled;
+    const size_t data_size = buff.data.size();
+    auto&        stream    = *st.stream.sam;
+
+    for(nb_filled = 0; nb_filled < data_size && stream.next() >= 0; ++nb_filled) {
+      ++reads_read_;
+      header_sequence_qual& fill_buff = buff.data[nb_filled];
+      fill_buff.header = stream.qname();
+      auto& seq  = fill_buff.seq;
+      auto& qual = fill_buff.qual;
+      seq.resize(stream.seq_len());
+      qual.resize(stream.seq_len());
+      for(ssize_t i = 0; i < stream.seq_len(); ++i) {
+        seq[i] = stream.base(i);
+        qual[i] = stream.qual(i) + '!';
+      }
+    }
+  }
+#endif
 };
 } // namespace jellyfish
 
